Class: JSON::Validator
- Inherits:
-
Object
- Object
- JSON::Validator
- Defined in:
- lib/json-schema/validator.rb
Constant Summary collapse
- @@schemas =
{}
- @@cache_schemas =
false- @@default_opts =
{ :list => false, :version => nil, :validate_schema => false }
- @@validators =
{}
- @@default_validator =
nil- @@available_json_backends =
[]
- @@json_backend =
'yajl'
Class Method Summary collapse
- .add_schema(schema) ⇒ Object
- .cache_schemas=(val) ⇒ Object
- .clear_cache ⇒ Object
- .default_validator ⇒ Object
- .json_backend ⇒ Object
- .json_backend=(backend) ⇒ Object
- .parse(s) ⇒ Object
- .register_default_validator(v) ⇒ Object
- .register_validator(v) ⇒ Object
- .schemas ⇒ Object
- .validate(schema, data, opts = {}) ⇒ Object
- .validate!(schema, data, opts = {}) ⇒ Object (also: validate2)
- .validators ⇒ Object
Instance Method Summary collapse
-
#build_schemas(parent_schema) ⇒ Object
Build all schemas with IDs, mapping out the namespace.
-
#handle_schema(parent_schema, obj) ⇒ Object
Either load a reference schema or create a new schema.
-
#initialize(schema_data, data, opts = {}) ⇒ Validator
constructor
A new instance of Validator.
- #load_ref_schema(parent_schema, ref) ⇒ Object
-
#validate ⇒ Object
Run a simple true/false validation of data against a schema.
Constructor Details
#initialize(schema_data, data, opts = {}) ⇒ Validator
Returns a new instance of Validator.
85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 |
# File 'lib/json-schema/validator.rb', line 85 def initialize(schema_data, data, opts={}) @options = @@default_opts.clone.merge(opts) # I'm not a fan of this, but it's quick and dirty to get it working for now version_string = "draft-03" if @options[:version] @options[:version] = case @options[:version].to_s when "draft3" "draft-03" when "draft2" "draft-02" when "draft1" "draft-01" else raise JSON::Schema::SchemaError.new("The requested JSON schema version is not supported") end version_string = @options[:version] u = URI.parse("http://json-schema.org/#{@options[:version]}/schema#") validator = JSON::Validator.validators["#{u.scheme}://#{u.host}#{u.path}"] @options[:version] = validator end # validate the schema, if requested if @options[:validate_schema] begin = File.join(Pathname.new(File.dirname(__FILE__)).parent.parent, "resources", "#{version_string}.json").to_s = JSON::Validator.new(, schema_data) .validate rescue JSON::Schema::ValidationError, JSON::Schema::SchemaError raise $! end end @base_schema = initialize_schema(schema_data) @data = initialize_data(data) build_schemas(@base_schema) end |
Class Method Details
.add_schema(schema) ⇒ Object
257 258 259 |
# File 'lib/json-schema/validator.rb', line 257 def add_schema(schema) @@schemas[schema.uri.to_s] = schema if @@schemas[schema.uri.to_s].nil? end |
.cache_schemas=(val) ⇒ Object
261 262 263 |
# File 'lib/json-schema/validator.rb', line 261 def cache_schemas=(val) @@cache_schemas = val == true ? true : false end |
.clear_cache ⇒ Object
249 250 251 |
# File 'lib/json-schema/validator.rb', line 249 def clear_cache @@schemas = {} if @@cache_schemas == false end |
.default_validator ⇒ Object
269 270 271 |
# File 'lib/json-schema/validator.rb', line 269 def default_validator @@default_validator end |
.json_backend ⇒ Object
281 282 283 |
# File 'lib/json-schema/validator.rb', line 281 def json_backend @@json_backend end |
.json_backend=(backend) ⇒ Object
285 286 287 288 289 290 291 292 |
# File 'lib/json-schema/validator.rb', line 285 def json_backend=(backend) backend = backend.to_s if @@available_json_backends.include?(backend) @@json_backend = backend else raise JSON::Schema::JsonParseError.new("The JSON backend '#{backend}' could not be found.") end end |
.parse(s) ⇒ Object
294 295 296 297 298 299 300 301 302 303 304 305 |
# File 'lib/json-schema/validator.rb', line 294 def parse(s) case @@json_backend.to_s when 'json' JSON.parse(s) when 'yajl' json = StringIO.new(s) parser = Yajl::Parser.new parser.parse(json) else raise JSON::Schema::JsonParseError.new("No supported JSON parsers found. The following parsers are suported:\n * yajl-ruby\n * json") end end |
.register_default_validator(v) ⇒ Object
277 278 279 |
# File 'lib/json-schema/validator.rb', line 277 def register_default_validator(v) @@default_validator = v end |
.register_validator(v) ⇒ Object
273 274 275 |
# File 'lib/json-schema/validator.rb', line 273 def register_validator(v) @@validators[v.to_s] = v end |
.schemas ⇒ Object
253 254 255 |
# File 'lib/json-schema/validator.rb', line 253 def schemas @@schemas end |
.validate(schema, data, opts = {}) ⇒ Object
232 233 234 235 236 237 238 239 240 |
# File 'lib/json-schema/validator.rb', line 232 def validate(schema, data,opts={}) begin validator = JSON::Validator.new(schema, data, opts) validator.validate return true rescue JSON::Schema::ValidationError, JSON::Schema::SchemaError return false end end |
.validate!(schema, data, opts = {}) ⇒ Object Also known as: validate2
242 243 244 245 246 |
# File 'lib/json-schema/validator.rb', line 242 def validate!(schema, data,opts={}) validator = JSON::Validator.new(schema, data, opts) validator.validate return true end |
.validators ⇒ Object
265 266 267 |
# File 'lib/json-schema/validator.rb', line 265 def validators @@validators end |
Instance Method Details
#build_schemas(parent_schema) ⇒ Object
Build all schemas with IDs, mapping out the namespace
174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 |
# File 'lib/json-schema/validator.rb', line 174 def build_schemas(parent_schema) # Build ref schemas if they exist if parent_schema.schema["$ref"] load_ref_schema(parent_schema, parent_schema.schema["$ref"]) end # Check for schemas in union types ["type", "disallow"].each do |key| if parent_schema.schema[key] && parent_schema.schema[key].is_a?(Array) parent_schema.schema[key].each_with_index do |type,i| if type.is_a?(Hash) handle_schema(parent_schema, type) end end end end # All properties are schemas if parent_schema.schema["properties"] parent_schema.schema["properties"].each do |k,v| handle_schema(parent_schema, v) end end # Items are always schemas if parent_schema.schema["items"] items = parent_schema.schema["items"].clone single = false if !items.is_a?(Array) items = [items] single = true end items.each_with_index do |item,i| handle_schema(parent_schema, item) end end # Each of these might be schemas ["additionalProperties", "additionalItems", "dependencies", "extends"].each do |key| if parent_schema.schema[key].is_a?(Hash) handle_schema(parent_schema, parent_schema.schema[key]) end end end |
#handle_schema(parent_schema, obj) ⇒ Object
Either load a reference schema or create a new schema
221 222 223 224 225 226 227 228 |
# File 'lib/json-schema/validator.rb', line 221 def handle_schema(parent_schema, obj) schema_uri = parent_schema.uri.clone schema = JSON::Schema.new(obj,schema_uri,@options[:version]) if obj['id'] Validator.add_schema(schema) end build_schemas(schema) end |
#load_ref_schema(parent_schema, ref) ⇒ Object
136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 |
# File 'lib/json-schema/validator.rb', line 136 def load_ref_schema(parent_schema,ref) uri = URI.parse(ref) if uri.relative? uri = parent_schema.uri.clone # Check for absolute path path = ref.split("#")[0] # This is a self reference and thus the schema does not need to be re-loaded if path.nil? || path == '' return end if path && path[0,1] == '/' uri.path = Pathname.new(path).cleanpath.to_s else uri = parent_schema.uri.merge(path) end uri.fragment = '' end if Validator.schemas[uri.to_s].nil? begin schema = JSON::Schema.new(JSON::Validator.parse(open(uri.to_s).read), uri, @options[:version]) Validator.add_schema(schema) build_schemas(schema) rescue JSON::ParserError # Don't rescue this error, we want JSON formatting issues to bubble up raise $! rescue Exception # Failures will occur when this URI cannot be referenced yet. Don't worry about it, # the proper error will fall out if the ref isn't ever defined end end end |
#validate ⇒ Object
Run a simple true/false validation of data against a schema
125 126 127 128 129 130 131 132 133 |
# File 'lib/json-schema/validator.rb', line 125 def validate() begin @base_schema.validate(@data,[]) Validator.clear_cache rescue JSON::Schema::ValidationError Validator.clear_cache raise $! end end |